(* :Copyright: Copyright 1997 Jens-Peer Kuska *)

(* :Title: PNGBitmap *)

(* :Context: PNGBitmap` *)

(* :Author: Jens-Peer Kuska *)

(* :Mathematica Version: 3.0 *)

(* :Package Version: 1.0 *)

(* :Keywords: Portable Network Graphics, Bitmap, Texture *)

(* :History: 
      - version 1.0 released with the pnglib 0.99d

*)

(* :Sources: None. *)

(* :Requirements: None. *)

(* :Limitations: *)

(* :Warnings: None. *)

(* :Summary:
This package provides functions that read and write
bitmaps of the Portable Network Graphics format into
the Kernel. A new graphics object PNGBitmap[] is defined.
*)

(* :Discussion: 

All kinds of PNG bitmaps can be read. Palette images expaned to 
gray or RGB(A) images. The alpha channel can't be seen inside
Mathematica but if a alpha channel is present it is saved and
can be manipulated with Mathematica. The Dispaly function get
a new format "PNG" to save graphics in png (RGB) format.
*)

(* :Examples: 	
See the documentation notebook.

Display[file,graph,"PNG"] write
*)

BeginPackage["PNGBitmap`",{"Utilities`FilterOptions`"}]

(* data structures *)

PNGBitmap::usage=
"PNGBitmap[image,width,height,bitdepth] represents a bitmap image. The image \
data stored in a matrix in the first argument."

Pixel::usage="Pixel[ri,gi,bi,alpha] represent a single pixel in a \
PNGBitmap[]-object. ri, gi, bi and alpha are red, green, blue and alpha \
channel of the pixel. Pixel[ri,gi,bi] repesent a RGB pixel, and Pixel[gray] \
a pixel of an gray scale image."

(* Input/Output *)

ReadPNGBitmap::usage="ReadPNGBitmap[pngfile] reads a Portable Network \
Graphics file into the kernel and returns a PNGBitmap[]-object."

WritePNGBitmap::usage="WritePNGBitmap[filename,bitmap,opts] \
writes the PNGBitmap[]-object bitmap to a png file with name filename.\n\
Possible options are:\n\
  PNGFormat one of {PNGColorRGBA, PNGColorRGB, PNGColorGray}\n\
  PNGGrayFunction  a function to convert RGB images to gray scale\n\
  PNGGamma a floating point number for gamma correction."

(* Utilities *)

RGBAColorImageQ::usage="RGBAColorImageQ[pngimg] returns True if the bitmap is a RGB alpha image."
			
RGBColorImageQ::usage="RGBColorImageQ[pngimg] returns True if the bitmap is a RGB image."

ColorImageQ::usage="ColorImageQ[pngimg] returns True if the bitmap is a color image."

GrayImageQ::usage="GrayImageQ[pngimg] returns True if the bitmap is a gray scale image image."

PNGBitmapToData::usage="PNGBitmapToData[bitmap,opts] convert the \
bitmap to a 2d array of data. For color images the option PNGGrayFunction can \
be given to convert RGB tripel to a single value."

DataToPNGBitmap::usage="DataToPNGBitmap[matrix,opt] will create a PNG bitmap from the numbers in matrix. \
The option PNGBitDepth -> 8 or PNGBitDepth-> 16 can be used to give a explicit \
bit depth for the resulting PNGBitmap-object."
PNGBitDepth::usage="PNGBitDepth is a option for DataToPNGBitmap[]."

RasterToPNGBitmap::usage="RasterToPNGBitmap[gr,opt] converts a RasterArray[] or a graphics \
with a single Graphics[RasterArray[..]] to a PNG bitmap. Note that RasterArrays[] with Hue[] \
directives are not converted.\n\
The only possible option is PNGBitDepth->8 or PNGBitDepth->16."

CountColors::usage="CountColors[bitmap] gives the number of unique colors in the bitmap."

ColorList::usage="ColorList[bitmap] gives a sorted list of unique colors in the image."

RandomPalette::usage="RandomPalette[bitmap] will replace the colors in a bitmap by random choosen RGB colors."

InvertBitmap::usage="InvertBitmap[bitmap] make the inverse bitmap."

MirrorVertical::usage="MirrorVertical[bitmap] mirrors the PNGBitmap[]-object vertical."

MirrorHorizontal::usage="MirrorHorizontal[bitmap] mirrors the PNGBitmap[]-object horizontal."

(* 3D Textures *)

PNGTexturePlot3D::usage="PNGTexturePlot3D[bitmap,fxy,{x,x1,x2},{y,y1,y2}, opts] \
makes a Plot3D[] of the function fxy and uses the bitmap as a texture \
on the surface. Usual the meshlines are omitted and fxy is compiled."

PNGTextureParametricPlot3D::usage=
"PNGTextureParametricPlot3D[bm,{fx,fy,fz},{u,u1,u2},{v,v1,v2},opts] \n
maps the bitmap bm on the three dimensional surface {fx[u,v],fy[u,v],fz[u,v]}. \
Beside the options for Graphics3D the options the options\n
DrawEdges         {True, EdgeForm[any]} \n\n
RGBColorFunction  a pure function to get from r,g,b and SurfaceColor[] or RGBColor[] \n
GrayColorFunction a pure function to get from a single gray value a SurfaceColor[] or GrayLevel[].\n\n\
are supported"

PNGMandelbrot::usage="PNGMandelbrot[{{x1,x2},{y1,y2}},opts] returns the Mandelbrot image as PNGBitamp. \
The options are\n 
PNGMakeJulia           make the Julia factal\n
PNGJuliaC              the complex number for the Julia fractal\n
PNGMandelScale         a pure function applied to every integer value used to form the gray levels.\n
PNGMandelBailOut,      the stoping bound for Abs[z[n]]\n
PNGMandelSize          the image size the default is {320,200}\n
PNGMandelIterations    the maximum number of iterations."

(* names *)

(#::usage=ToString[#]<>" is a symbol for indication of the color type of a PNGBitmap[]-object.") & /@ 
  {PNGColorRGBA, PNGColorRGB, PNGColorGray, PNGColorUnkown,PNGColorGrayA,PNGColorHue1};

(#::usage=ToString[#]<>" is a option for WritePNGBitmap[].") & /@ 
{PNGFormat, PNGGrayFunction, PNGGamma}

(#::usage=ToString[#]<>" is a option for PNGTextureParametricPlot3D[].") & /@ 
  {DrawEdges, RGBColorFunction, GrayColorFunction }

(#::usage=ToString[#]<>" is a option for PNGMandelbrot[].") & /@ 
   {PNGMandelScale, PNGMakeJulia,PNGJuliaC, PNGMandelBailOut, PNGMandelSize, PNGMandelIterations,PNGMandelPotential}


PNGColorPalette::usage="PNGColorPalette is used internal."

Begin["`Private`"]

sysname[]=
  Module[{sys},
    sys=$System;
    If[StringMatchQ[sys,"*Windows*"],
      "Windows",
      sys
     ]
    ]
        
mlprogname=First[
             Flatten[
               Select[
                 Join[
                   FileNames["mathpng",{#}],
				   FileNames["mathpng.exe",{#}],
                   FileNames[
                     "mmapng*",
                      {# <> $PathnameSeparator<> "bin"<> $PathnameSeparator <> 
                      sysname[]},
                    1
                   ]
                  ]  & /@ $Path,
                 #!={} &]
              ]
            ];

If[!(Or @@ (StringMatchQ[First[#],"*mathpng*"] & /@ Links[])), (* Then *)
    pnglink=Install[mlprogname]
 ] 

ReadPNGBitmap[pngfile_String]:=
  Module[{res,bm,w,h,dpt,type,wb},
    res=PNGBitmap`RawReadPNGBitmap[pngfile];
    If[res===$Failed || res==$Aborted,Return[$Failed]];
    {bm,w,h,dpt,type}=res;
    wb=Last[Dimensions[bm]]/w;
    If[wb==2, type=PNGColorGrayA];
    (* we had expanded the RGB and palette pictures to full RGBA entries *)
    If[MemberQ[{PNGColorPalette,PNGColorRGB,PNGColorRGBA},type], (* Then *)
      PNGBitmap @@ {
        Map[
	  Apply[Pixel,#] &,
	  Partition[#,4] &/@ bm,
	  {2}],w,h,dpt},
    (* Else *)	  
      If[type===PNGColorGrayA,(* Then, strip the alpha channel for bitmaps *)
        PNGBitmap @@ {
          Map[
	    Pixel[First[#]] &,
	    Partition[#,2] & /@ bm,{2}
	   ],w,h,dpt
	  },
	(* Else *)  
        PNGBitmap @@ {Map[Pixel,bm,{2}],w,h,dpt}
       ]	
     ]
   ]
		
(* Test the types of images *)
Clear[RGBAColorImageQ,RGBColorImageQ,ColorImageQ,GrayImageQ]

RGBAColorImageQ[PNGBitmap[img_,__]]:=RGBAColorImageQ[img]
RGBAColorImageQ[img_]:=MatchQ[img,{{Pixel[_,_,_,_]..}..}]
			

RGBColorImageQ[PNGBitmap[img_,__]]:=RGBColorImageQ[img]
RGBColorImageQ[img_]:=MatchQ[img,{{Pixel[_,_,_]..}..}]

ColorImageQ[PNGBitmap[img_,__]]:=ColorImageQ[img]
ColorImageQ[img_]:=MatchQ[img,{{Pixel[_,_,__]..}..}]

GrayImageQ[PNGBitmap[img_,__]]:=GrayImageQ[img]
GrayImageQ[img_]:=MatchQ[img,{{Pixel[_]..}..}]


PNGBitmap /: 
  Graphics[PNGBitmap[im_,w_Integer,h_Integer,bd_ /; bd<=16],opts___?OptionQ]:=
    Module [{ww,hh,fac,raster},
      {hh,ww}=Dimensions[im];
      If[ww=!=w || hh=!=h,
  	    Return[Graphics[Text["Corrupted PNG Image",{0,0}],opts]]     
       ];
	  fac=N[1.0/(2^bd-1)];
	  Graphics[
	    RasterArray[
	      im /. {
			     Pixel[g_] :> GrayLevel[fac*g],
	             Pixel[r_,g_,b_,___]:> 
				   RGBColor[Sequence @@ (#*fac & /@ {r,g,b})]
			    }
	     ],
	   AspectRatio->hh/ww,opts]
	 ]


PNGBitmap /: Display[channel_,bm_PNGBitmap,fmt___]:=
               Display[channel,Graphics[bm],fmt]

CountColors[PNGBitmap[bm_,__]]:=Length[Union[Flatten[bm]]]

ColorList[PNGBitmap[bm_,__]]:=Sort[Union[Flatten[bm]]]

RandomPalette[bm_PNGBitmap]:=
  Module[{cl,n,tocol,bd,rules},
    bd=2^Last[bm]-1;
    cl=ColorList[bm];
    n=Length[cl]; 
    tocol= Sort[
             Table[
               Pixel[
                 Random[Integer,bd],
                 Random[Integer,bd],
                 Random[Integer,bd]
                ],{i, 1, n}
              ]
            ]; 
     rules= Dispatch[Thread[cl->tocol]];
     bm /. rules        
    ]

PNGBitmap /: Transpose[PNGBitmap[bm_,w_,h_,bd_]]:= 
  PNGBitmap @@ {Transpose[bm],h,w,bd}

InvertBitmap[PNGBitmap[bm_,w_,h_,bd_]]:=
  Module[{bmax},
    bmax=2^bd-1;
    PNGBitmap @@ {
      bm /. {
              Pixel[r_,g_,b_,a___]:> Pixel[bmax-r,bmax-g,bmax-b,a],
              Pixel[g_] :> Pixel[bmax-g]
             }, 
      w,h,bd
      }
   ]     

MirrorVertical[PNGBitmap[bm_,w_,h_,bd_]]:=PNGBitmap@@ { Reverse /@ bm,w,h,bd}

MirrorHorizontal[PNGBitmap[bm_,w_,h_,bd_]]:=PNGBitmap @@ {Reverse[bm],w,h,bd}

Options[WritePNGBitmap]:={
    PNGFormat->Automatic,
    PNGGrayFunction->(Apply[Plus,#]/3 &),
    PNGGamma->1.0
  };

$PNGFormatsSupported={PNGColorGray,PNGColorRGB,PNGColorRGBA};

Clear[PNGGetFormat]

PNGGetFormat[PNGBitmap[im_,__]]:=PNGGetFormat[im]

PNGGetFormat[im_?RGBAColorImageQ]:=PNGColorRGBA

PNGGetFormat[im_?RGBColorImageQ]:=PNGColorRGB

PNGGetFormat[im_?GrayImageQ]:=PNGColorGray

PNGGetFormat[im_]:=PNGColorUnkown

WritePNGBitmap::uformat="Unkown format type `1` given. Use one of `2`.";

WritePNGBitmap::noreal="PNGGamma must be a real number, instead of `1`";

WritePNGBitmap::invalidpic="Invalid Pixel[] format found.";

WritePNGBitmap[pngfile_String,PNGBitmap[img_,w_,h_,bd_],opts___?OptionQ]:=
  Module[{gamma,fmt,grfun,image,fcode},
    gamma=N[PNGGamma /. {opts} /. Options[WritePNGBitmap]] /. 0->0.0;
    fmt=PNGFormat /. {opts} /. Options[WritePNGBitmap];
    If[fmt===Automatic,fmt=PNGGetFormat[img]];
    If[!MemberQ[$PNGFormatsSupported,fmt],
      Message[WritePNGBitmap::uformat,fmt,$PNGFormatsSupported];
      Return[$Failed]
     ];		
    grfun=PNGGrayFunction /. {opts} /. Options[WritePNGBitmap];
    If[!NumberQ[gamma],
      Message[WritePNGBitmap::noreal,gamma];
      Return[$Failed]
     ];
    Switch[fmt,
      PNGColorGray, 
	 image=img  /. 
	         { Pixel[r_Integer,g_Integer,b_Integer,_]:>
		     grfun[{r,g,b}],
                   Pixel[g_]:>g 
		  };
	 fcode=1,
      PNGColorRGB, 
	 image= img  /. 
           {Pixel[r_Integer,g_Integer,b_Integer,___]:>
	     Sequence[r,g,b], Pixel[g_]:> Sequence[g,g,g]};
	 fcode=3,
      PNGColorRGBA,
	 image=img /.  {Pixel[r_Integer,g_Integer,b_Integer,a_]:>
	                  Sequence[r,g,b,a],
			Pixel[r_Integer,g_Integer,b_Integer,]:>
	                  Sequence[r,g,b,255], 
			Pixel[g_]:> Sequence[g,g,g,255]};

	 fcode=4,
      _,
          image=img /. Pixel[rgba__]:> rgba;
	  fcode=4
     ];
    If[!MatrixQ[image,IntegerQ[#] && (2^bd >#) &],
      Message[WritePNGBitmap::invalidpic];
      Return[$Failed]
     ];			
    {PNGBitmap`RawWritePNGBitmap[pngfile,w,h,bd,fcode,gamma,image],fmt}
   ]
			
(* Write RasterArray[] and Graphics[RasterArray[]] direct *)

WritePNGBitmap[pngfile_String,r_RasterArray,opts___?OptionQ]:=
  WritePNGBitmap[pngfile,RasterToPNGBitmap[r],opts]

WritePNGBitmap[pngfile_String,Graphics[r_RasterArray,___],opts___?OptionQ]:=
  WritePNGBitmap[pngfile,RasterToPNGBitmap[r],opts]

WritePNGBitmap[pngfile_String,Graphics[{r_RasterArray},___],opts___?OptionQ]:=
  WritePNGBitmap[pngfile,RasterToPNGBitmap[r],opts]  

Options[PNGBitmapToData]:={PNGGrayFunction->(Apply[Plus,#]/3 &)}

PNGBitmapToData[
  PNGBitmap[im_,w_,h_,d_] /; ColorImageQ[im],
  opts___?OptionQ]:=
  Module[{image,tgrayfun},
    tgrayfun=PNGGrayFunction /. {opts} /.Options[PNGBitmapToGrayData];
    im  /. Pixel[r_Integer,g_Integer,b_Integer,___]:>tgrayfun[{r,g,b}]
   ]

PNGBitmapToData[PNGBitmap[im_,_,_,_] /; GrayImageQ[im],opts___]:=
  im /. Pixel[g_]:> g

Options[DataToPNGBitmap]:={PNGBitDepth->8};

DataToPNGBitmap[data_?(MatrixQ[#,NumericQ] &),opts___]:=
	  Module[{bd,fac,dim,dmin,dmax},
		  bd=PNGBitDepth /. {opts} /. Options[DataToPNGBitmap];
		  If[bd<8,bd=8];
		  If[bd>8 && bd<16,bd=16];
		  dim=Reverse[Dimensions[data]];
		 {dmin,dmax}={Min[#],Max[#]} &[Flatten[data]];
		  fac=(2^bd-1)/(dmax-dmin);
		  PNGBitmap @@ {Map[Pixel[Floor[N[fac*(#-dmin)]]] &,data,{2}],
                                Sequence @@ dim,bd}
		]
(* We want write Mathematicas representation of
   bit maps as RasterArray[] as well *)

getRasterFormat[r_ ] /; MatchQ[r,{{_RGBColor..}..}]:=PNGColorRGB
getRasterFormat[r_] /; MatchQ[r,{{_GrayLevel..}..}]:=PNGColorGray
getRasterFormat[r_] /; MatchQ[r,{{Hue[__]..}..}]:=PNGColorHue1
getRasterFormat[r_] :=PNGColorUnkown

(* Convert RasterArray[] with Hue[_] directives  *)

HueToRGB[fac_Integer,h_?NumericQ,0,b_:1] /; h<0.0 :=RGBColor[b,b,b]
HueToRGB[fac_Integer,h_?NumericQ,s_:1,v_:1]:=
  Module[{hh,i,f,p,q,t,trip},
    hh=Mod[h,1.0];
    If[hh<0.0, hh+=1.0];
    hh*=6.0;
    i=Floor[hh];
    f=hh-i;
    p=v*(1.0-s);
    q=v*(1.0-s*f);
    t=v*(1.0-s*(1.0-f));
    Switch[i,
      0, trip={v,t,p},
      1, trip={q,v,p},
      2, trip={p,v,t},
      3, trip={p,q,v},
      4, trip={t,p,v},
      5, trip={v,p,q},
      _, trip={0,0,0}
    ];
    Pixel @@ (Floor[fac*N[#]]& /@ trip)
   ]

Options[RasterToPNGBitmap]:=
	{PNGBitDepth->8}

RasterToPNGBitmap[r_RasterArray,opts___?OptionQ]:=
  Module[{bd,fmt,dim,bm,fac},
    bd=PNGBitDepth /. {opts} /. Options[RasterToPNGBitmap];
	If[bd<8,bd=8];
	If[bd>8 && bd<16, bd=16];
    fac=2^bd-1;
    bm=First[r]; (* take only the array *)
	dim=Reverse[Dimensions[bm]];
	fmt=getRasterFormat[bm];
	If[MemberQ[{PNGColorRGB,PNGColorGray,PNGColorHue1},fmt],
	  PNGBitmap @@ {
        bm /. {(GrayLevel | RGBColor)[c__]:> Pixel@@ (Floor[fac*# & /@ {c}]),
		       Hue[c__] :> HueToRGB[fac,c]},
        Sequence @@ dim,bd},
	  $Failed
	 ]
   ]

(* Convert graphics with a single RasterArray[] *)

RasterToPNGBitmap[Graphics[r_RasterArray,___],opts___]:=
  RasterToPNGBitmap[r,opts]

RasterToPNGBitmap[Graphics[{r_RasterArray},___],opts___]:=
  RasterToPNGBitmap[r,opts]

(* The portable bitmap format is used to create a 
   new format for the Display[] function. The
   ImageSize option is used to specify the size
   of the graphics. *)

Unprotect[Display]

Display[file_String,gr_,"PNG"]:=
  (PNGBitmap`RawWritePNGGraphics[file,1.0,DisplayString[gr,"PBM"]];gr)

Protect[Display]

(* We don't want to enter the bitmap by hand but the remaining 
   arguments are keept unevaluated.                            *)
   
SetAttributes[PNGTexturePlot3D,HoldRest]

Options[PNGTexturePlot3D]:={Compiled->True,Mesh->False}

PNGTexturePlot3D[bm_PNGBitmap,fun_,
  {x_,x1_?NumericQ,x2_?NumericQ},
  {y_,y1_?NumericQ,y2_?NumericQ}, opts___?OptionQ]:=
  Module[{dta,i,j,options,comp,f3d,bd,dx,dy,nx,ny,drwmesh},
    comp=Compiled /. {opts} /. Options[PNGTexturePlot3D];   
    If[comp,
      f3d=Compile[{x,y},fun],
      f3d=Function[{x,y},fun]
     ];		
    drwmesh=Mesh /. {opts} /. Options[PNGTexturePlot3D]; 	
    nx=bm[[2]];
    ny=bm[[3]];
    dx=N[(x2-x1)/nx];
    dy=N[(y2-y1)/ny];
    data=Table[f3d[x1+i*dx,y1+j*dy],{j,0,ny},{i,0,nx}];
    bd=N[2^Last[bm]-1];
    options=Select[
              {opts},
              !MemberQ[
                 {Mesh,MeshRange,Compiled},
                 First[#]] &
             ];
    ListPlot3D[
      data,
      bm[[1]] /. 
	{Pixel[g_]:>GrayLevel[N[g/bd]],
	 Pixel[r_,g_,b_,___]:>RGBColor[r/bd,g/bd,b/bd]},
      Mesh->drwmesh,
      MeshRange->{{x1,x2},{y1,y2}} , 
      FilterOptions[ListPlot3D,Sequence @@ options]
     ]
    ]

(* 3D parametric plots *)

SetAttributes[PNGTextureParametricPlot3D,HoldRest]

Options[PNGTextureParametricPlot3D]:={
  Compiled->True, 
  DrawEdges->EdgeForm[],
  RGBColorFunction->(SurfaceColor[RGBColor[##]] &),
  GrayColorFunction->(SurfaceColor[GrayLevel[##]] &)
 }


makeMesh[dta_]:=
  Module[{mesh,rdta},
    rdta=RotateLeft /@ dta;
    mesh={dta, rdta,RotateLeft[rdta],RotateLeft[dta]};
    mesh=Map[Drop[#,-1] &,mesh,{1}];
    mesh=Map[Drop[#,-1] &,mesh,{2}];
    Map[
      Polygon ,
      Transpose[ Flatten [#,1] & /@ mesh ]
     ]
   ]

PNGTextureParametricPlot3D[
  bm_PNGBitmap,
  f:{_,_,_},
  {u_,u1_?NumericQ,u2_?NumericQ},{v_,v1_?NumericQ,v2_?NumericQ},
  opts___]:=
  Module[{nu,nv,du,dv,dta,comp,i,j,bd,edge,cf,gf},
    comp=Compiled /. {opts} /. Options[PNGTextureParametricPlot3D];
    If[comp,
      fun=Compile[{u,v},f],
      fun=Function[{u,v},f]
     ];
    edge=DrawEdges /. {opts} /. Options[PNGTextureParametricPlot3D];
    If[edge===False, edge=EdgeForm[]];
    If[Head[edge]=!=EdgeForm, edge=Sequence[]];
    cf=RGBColorFunction /. {opts} /. Options[PNGTextureParametricPlot3D];
    gf=GrayColorFunction/. {opts} /. Options[PNGTextureParametricPlot3D];
    nu=bm[[2]];
    nv=bm[[3]];
    bd=N[2^Last[bm]-1];
    du=N[(u2-u1)/nu];
    dv=N[(v2-v1)/nv];
    dta=Table[fun[u1+i*du,v1+j*dv],{j,0,nv},{i,0,nu}];
    dta={edge,
	  Transpose[
            {
              Flatten[bm[[1]]] /.   
                {Pixel[r_,g_,b_,a__]:> 
                   Evaluate[cf][r/bd,g/bd,b/bd],
                 Pixel[g_]:> Evaluate[gf][g/bd]},
	     makeMesh[dta]}
	    ]
        };
     Show[Graphics3D[dta,FilterOptions[Graphics3D,opts]]]
    ]

(* some popular applications of bitmap graphics *)

Options[PNGMandelbrot]:={PNGMandelScale->(# &),
		         PNGMandelBailOut->100.0,
		         PNGMandelSize->{320,200},
		         PNGMandelIterations->255,
				 PNGMandelPotential->True,
				 PNGMakeJulia->False,
				 PNGJuliaC->0.3+I*0.6}
		         
PNGMandelbrot::noint="`1` must be an integer instead of `2`"
PNGMandelbrot::nonum="`1` must be a number instead of `2`"

PNGMandelbrot[
  {{x1_?NumericQ,x2_?NumericQ},
  {y1_?NumericQ,y2_?NumericQ}},opts___]:=
	PNGMandelbrot[x1,y1,x2,y2,opts]

PNGMandelbrot[x1_?NumericQ, y1_?NumericQ, x2_?NumericQ, y2_?NumericQ, opts___?OptionQ]:=
  Module[{img,xsize,ysize,bailout,maxit,
          scfun,minZ,maxZ,dZ,bd,fac,usePot,
		  mjulia,juliac},
    {xsize,ysize}=PNGMandelSize /. {opts} /. Options[PNGMandelbrot];
    bailout=PNGMandelBailOut /. {opts} /. Options[PNGMandelbrot];
    maxit= PNGMandelIterations /. {opts} /. Options[PNGMandelbrot];
    scfun=PNGMandelScale /. {opts} /. Options[PNGMandelbrot];
	usePot=PNGMandelPotential /. {opts} /. Options[PNGMandelbrot];
	mjulia=PNGMakeJulia  /. {opts} /. Options[PNGMandelbrot];
	juliac=PNGJuliaC  /. {opts} /. Options[PNGMandelbrot];
    If[!IntegerQ[xsize],
      Message[PNGMandelbrot::noint,"xsize",xsize];
      Return[$Failed]
     ];
    If[!IntegerQ[ysize],
      Message[PNGMandelbrot::noint,"ysize",ysize];
      Return[$Failed]
     ];
    If[!IntegerQ[maxit],
      Message[PNGMandelbrot::noint,"maximum iterations",maxit];
      Return[$Failed]
     ];
    If[!NumericQ[bailout],
      Message[PNGMandelbrot::nonum,"Bail out",bailout];
      Return[$Failed]
     ];
	If[usePot===True,usePot=1,usePot=0];
	If[mjulia===True,mjulia=1,mjulia=0];
	If[!NumericQ[juliac],
	  Message[PNGMandelbrot::nonum,"PNGJuliaC",juliac];
	  Return[$Failed]
	 ];
    img=PNGBitmap`RawMandelJulia@@ Join[
                      N[{x1,y1,x2,y2,Re[juliac],Im[juliac],bailout}] /. 0->0.0,
                      {mjulia,xsize,ysize,maxit,usePot}
                     ];
    img=N[Map[scfun,img,{2}]];
    {minZ,maxZ}={Min[#],Max[#]} & @(Flatten[img]);
    dZ=(maxZ-minZ);
    If[maxit<2^8 && 0==usePot,
      bd=8;
      fac=(2^8-1)/dZ,
    (* Else *)  
      bd=16;
      fac=(2^16-1)/dZ
     ]; 
    PNGBitmap @@ {Map[Pixel[Round[fac*(#-minZ)]] &,img,{2}],xsize,ysize,bd}
   ]	




End[] (* Private *)
EndPackage[]